#include <iostream.h>
#include <iomanip.h>

#define  matrixDimension 3 
#define  noElement 27534223.9787 // A number which is highly unlikely to be used. 
enum orientation{ABOVE_PE,BELOW_PE,RIGHT_PE,LEFT_PE} ;
// Definition of PEs
// Generic PE
class PE
{
public:
	PE()
	{
		outputUpdated=false;
		pInterfaceMatrixElement=NULL;
		residentData=noElement;
		readyForCalculation=false;
	}
	~PE() {}
// Since the neighboring PEs' types are not known in the general case, this virtual function does the
// run-time type identification task and deligates the execution to the proper function.
	virtual double sendOutput(int)
	{
		cout<<"This function won't get called ." ;
		return NULL;
	}
	double showResidentData() {	return residentData;}
	void outputResidentData()
	{
		if (residentData!=noElement)
			cout<<" *Resident Data:"<<residentData ;
		else
			cout<<" *Resident Data:"<<"No Element" ;
	}
	int showMode() { return mode;}

	int outputNumber;
// This variable is set to true after performing any calculation in the PEs.  
	bool outputUpdated;
// This variable is set to true after reading the inputs, if any, in the PEs.  
	bool readyForCalculation;
// If the pointer to PE, in each PE, is set to NULL then pInterfaceMatrixElement will be used.
// Applicable to outer PEs in the CSA
	double * pInterfaceMatrixElement;
protected:
	double residentData;
	int mode ;
};

class EllipticPE : public PE
{
public:
	EllipticPE(){
		rightEndOfTheGrid=false;
		m1Input1=noElement;
		m1Input2=noElement;
		m2Input1=noElement;
		m2Input2=noElement;
		m1Output1=noElement;
		m1Output2=noElement;
		m2Output1=noElement;
		m2Output2=noElement;
	}
	~EllipticPE() {}
	void readInput() ;
	virtual double sendOutput(int) ;
	void doClockCalculations() ; 
	double outputM2output1(){ return m2Output1;}
	void setMode(int inputMode);
	void showInputVariables();
	void showOutputVariables();
//it is used to set the PE which is at the right end of the grid
	void setRightEndOfTheGrid(bool temp) {rightEndOfTheGrid=temp;}
// Pointers that designate the Above/Below/Right/Left PE of the current Elliptic PE
	EllipticPE *  pAbovePE ;
	PE *  pBelowPE ;
	PE *  pRightPE ;
	PE *  pLeftPE ;

private:
	bool rightEndOfTheGrid;
	// Mode 1 input variables  :
	double m1Input1;
	double m1Input2;
	// Mode 1 output variables  :
	double m1Output1;
	double m1Output2;
	// Mode 2 input variables :
	double m2Input1 ;
	double m2Input2 ;
	// Mode 2 output variables :
	double m2Output1 ;
	double m2Output2 ;
};

void EllipticPE::readInput() 
	{
		if (mode==1)
		{
			if (residentData!=noElement)
			{
				// Applicable to inner PEs
				if (pAbovePE!=NULL)
				{
					// Check to see whether the inputs are updated, and if so proceed with getting
					// the inputs and doing the calculations. "outputUpdated" variable is also set.
					if ( (pAbovePE->outputUpdated==true) && (pLeftPE->outputUpdated==true) )
					{
						m1Input1= pAbovePE->sendOutput(BELOW_PE) ;
						m1Input2= pLeftPE->sendOutput(RIGHT_PE) ;

					pAbovePE->outputNumber=pAbovePE->outputNumber-1;
					if (pAbovePE->outputNumber==0)
						pAbovePE->outputUpdated=false;

					pLeftPE->outputNumber=pLeftPE->outputNumber-1;
					if (pLeftPE->outputNumber==0)
						pLeftPE->outputUpdated=false;

						readyForCalculation=true;
					} 
					else
					{
						m1Input1=noElement;
						m1Input2=noElement;
					}
				}
	            //Applicable to outer PEs for which the input matrix and unity matrix 
				//are also the potential input providers 
				else  
				{
					// Check to see whether the inputs are updated, and if so proceed with getting
					// the inputs and doing the calculations. "outputUpdated" variable is also set.
					if ( (*pInterfaceMatrixElement !=noElement) && (pLeftPE->outputUpdated==true) )
					{
						m1Input1= *pInterfaceMatrixElement;
						m1Input2= pLeftPE->sendOutput(RIGHT_PE) ;

					pLeftPE->outputNumber=pLeftPE->outputNumber-1;
					if (pLeftPE->outputNumber==0)
						pLeftPE->outputUpdated=false;

						readyForCalculation=true;
					}
					else
					{
						m1Input1=noElement;
						m1Input2=noElement;
					}
				}
			}
			else  // (residentData==noElement) is true
			{
				// Applicable to inner PEs
				if (pAbovePE!=NULL)
				{
					// Check to see whether the input is updated, and if so proceed with getting
					// the input and setting the residentData
					if (pAbovePE->outputUpdated==true)
					{
						m1Input1= pAbovePE->sendOutput(BELOW_PE) ;

					pAbovePE->outputNumber=pAbovePE->outputNumber-1;
					if (pAbovePE->outputNumber==0)
						pAbovePE->outputUpdated=false;

						residentData=m1Input1;
					} 
					else
						m1Input1=noElement;
				}
	            //Applicable to outer PEs for which the main matrix and unity matrix 
				//are also the potential input providers 
				else  
				{
					// Check to see whether the input is updated, and if so proceed with getting
					// the input and setting the residentData
					if (*pInterfaceMatrixElement !=noElement)
					{
						m1Input1= *pInterfaceMatrixElement;
						residentData=m1Input1;
					}
					else
						m1Input1=noElement;
				}
			}
		}
		else // ( 2nd mode )
		{
			// Check to see whether the inputs are updated, and if so proceed with getting
			// the inputs and doing the calculations. "outputUpdated" variable is also set.
			if ( (pRightPE->outputUpdated==true) && (pBelowPE->outputUpdated==true) )
				{
					m2Input1= pBelowPE->sendOutput(ABOVE_PE) ;
					m2Input2= pRightPE->sendOutput(LEFT_PE) ;

					pBelowPE->outputNumber=pBelowPE->outputNumber-1;
					if (pBelowPE->outputNumber==0)
						pBelowPE->outputUpdated=false;

					pRightPE->outputNumber=pRightPE->outputNumber-1;
					if (pRightPE->outputNumber==0)
						pRightPE->outputUpdated=false;

					readyForCalculation=true;
				} 
			else
			{
				m2Input1=noElement;
				m2Input2=noElement;
			}
		}
}

double EllipticPE::sendOutput(int outputDirection )
{
	if (mode==1)
	{
		if(outputDirection==BELOW_PE)
			return m1Output1 ;
		if(outputDirection==RIGHT_PE)
			return m1Output2;
	}
	else 
	{
		if(outputDirection==ABOVE_PE)
			return m2Output1 ;
		if(outputDirection==LEFT_PE)
			return m2Output2;
	}
}


void EllipticPE::doClockCalculations() 
{
	if (readyForCalculation==true)
	{
		if (mode==1)
		{
			m1Output1=m1Input1-(m1Input2* residentData);
			m1Output2=m1Input2;
			outputUpdated=true;
			readyForCalculation=false;
		}
		else
		{
			m2Output1=m2Input1;
			m2Output2=m2Input2-(m2Input1* residentData);
			outputUpdated=true;
			readyForCalculation=false;
		}
	}
	else
	{
		m1Output1=noElement;
		m1Output2=noElement;
		m2Output1=noElement;
		m2Output2=noElement;
	}
}
void EllipticPE::setMode(int inputMode)
	{
		mode=inputMode;
		outputNumber=2;
		if( (pAbovePE==NULL) && (mode==2) )
			outputNumber=1;
		if ( (rightEndOfTheGrid==true) && (mode==1) )
 			outputNumber=1;
	}	
void EllipticPE::showInputVariables()
	{
		if (mode==1)
		{
			if (m1Input1!=noElement)
				cout<<" *Input 1:"<<m1Input1 ;
			else
				cout<<" *Input 1:"<<"No Element" ;
			if (m1Input2!=noElement)
				cout<<" *Input 2:"<<m1Input2 ;
			else
				cout<<" *Input 2:"<<"No Element" ;
		}
		else
		{
			if (m2Input1!=noElement)
				cout<<" *Input 1:"<<m2Input1 ;
			else
				cout<<" *Input 1:"<<"No Element" ;
			if (m2Input2!=noElement)
				cout<<" *Input 2:"<<m2Input2 ;
			else
				cout<<" *Input 2:"<<"No Element" ;
		}
	}
void EllipticPE::showOutputVariables()
	{
		if (mode==1)
		{
			if (m1Output1!=noElement)
				cout<<" *Output 1:"<<m1Output1 ;
			else
				cout<<" *Output 1:"<<"No Element" ;
			if (m1Output2!=noElement)
				cout<<" *Output 2:"<<m1Output2 ;
			else
				cout<<" *Output 2:"<<"No Element" ;
		}
		else
		{
			if (m2Output1!=noElement)
				cout<<" *Output 1:"<<m2Output1 ;
			else
				cout<<" *Output 1:"<<"No Element" ;
			if (m2Output2!=noElement)
				cout<<" *Output 2:"<<m2Output2 ;
			else
				cout<<" *Output 2:"<<"No Element" ;
		}
	}

class SquarePE : public PE
{
public:
	SquarePE()
	{
		m1Input=noElement;
		m2Input=noElement;
		m1Output=noElement;
		m2Output=noElement;
	}
	~SquarePE() {}
	void readInput() ;
	virtual double sendOutput(int) ;
	void doClockCalculations() ; 
	double outputM2output(){ return m2Output;}
	void setMode(int inputMode);
	void showInputVariables();
	void showOutputVariables();
// Pointers that designate the Above/Right PE of the current Square PE
	EllipticPE *  pAbovePE ;
	PE *  pRightPE ;
private:
	// Mode 1 input variable  :
	double m1Input;
	// Mode 1 output variable  :
	double m1Output;
	// Mode 2 input variable :
	double m2Input ;
	// Mode 2 output variable :
	double m2Output ;
};

void SquarePE::readInput()
{
	if (mode==1)
		{
			if (residentData!=noElement)
			{
				// Applicable to inner PEs
				if (pAbovePE!=NULL)
				{
					// Check to see whether the input is updated, and if so proceed with getting
					// the input and doing the calculations. "outputUpdated" variable is also set.
					if(pAbovePE->outputUpdated==true)
					{
						m1Input= pAbovePE->sendOutput(BELOW_PE) ;

					pAbovePE->outputNumber=pAbovePE->outputNumber-1;
					if (pAbovePE->outputNumber==0)
						pAbovePE->outputUpdated=false;

						readyForCalculation=true;
					} 
					else
						m1Input=noElement;
				}
	            //Applicable to outer PEs for which the main matrix and unity matrix 
				//are also the potential input providers 
				else  
				{
					// Check to see whether the inputs are updated, and if so proceed with getting
					// the inputs and doing the calculations. "outputUpdated" variable is also set.
					if (*pInterfaceMatrixElement !=noElement)
					{
						m1Input= *pInterfaceMatrixElement;
						readyForCalculation=true;
					}
					else
						m1Input=noElement;
				}
			}
			else  // (residentData==noElement) is true
			{
				// Applicable to inner PEs
				if (pAbovePE!=NULL)
				{
					// Check to see whether the input is updated, and if so proceed with getting
					// the input and setting the residentData
	
					if (pAbovePE->outputUpdated==true)
					{
						m1Input= pAbovePE->sendOutput(BELOW_PE) ;

					pAbovePE->outputNumber=pAbovePE->outputNumber-1;
					if (pAbovePE->outputNumber==0)
						pAbovePE->outputUpdated=false;

						residentData=m1Input;
					} 
					else
						m1Input=noElement;
				}
	            //Applicable to outer PEs for which the main matrix and unity matrix 
				//are also the potential input providers 
				else  
				{
					// Check to see whether the input is updated, and if so proceed with getting
					// the input and setting the residentData
	
					if (*pInterfaceMatrixElement !=noElement)
					{
						m1Input= *pInterfaceMatrixElement;
						residentData=m1Input;
					}
					else
						m1Input=noElement;
				}
			}
		}
		else // ( 2nd mode )
		{
			// Check to see whether the inputs are updated, and if so proceed with getting
			// the inputs and doing the calculations. "outputUpdated" variable is also set.
			if (pRightPE->outputUpdated==true)
				{
					m2Input= pRightPE->sendOutput(LEFT_PE) ;

					pRightPE->outputNumber=pRightPE->outputNumber-1;
					if (pRightPE->outputNumber==0)
						pRightPE->outputUpdated=false;

					readyForCalculation=true;
				}
			else
				m2Input=noElement;
		}
}

double SquarePE::sendOutput(int)
{
	if (mode==1)
		return m1Output;
	else 
		return m2Output;
}

void SquarePE::doClockCalculations()
{
	if (readyForCalculation==true)
	{
		if (mode==1)
		{
			m1Output=m1Input/residentData;
			outputUpdated=true;
			readyForCalculation=false;
		}
		else
		{
			m2Output=m2Input/residentData;
			outputUpdated=true;
			readyForCalculation=false;
		}
	}
}
void SquarePE::setMode(int inputMode)
	{
		mode=inputMode;
		if (mode==1)
			outputNumber=2;
		else
		{
			outputNumber=1;
			if (pAbovePE==NULL) 
			outputNumber=0;
		}
	}
void SquarePE::showInputVariables()
	{
		if (mode==1)
		{
			if (m1Input!=noElement)
				cout<<" *Input:"<<m1Input;
			else
				cout<<" *Input:"<<"No Element" ;
		}
		else
		{
			if (m2Input!=noElement)
				cout<<" *Input:"<<m2Input;
			else
				cout<<" *Input:"<<"No Element" ;
		}
	}
void SquarePE::showOutputVariables()
	{
		if (mode==1)
		{
			if (m1Output!=noElement)
				cout<<" *Output:"<<m1Output;
			else
				cout<<" *Output:"<<"No Element" ;
		}
		else
		{
			if (m2Output!=noElement)
				cout<<" *Output:"<<m2Output;
			else
				cout<<" *Output:"<<"No Element" ;
		}
	}

class CircularPE : public PE
{
public:
	CircularPE(){
		m1Input1=noElement;
		m1Input2=noElement;
		m2Input1=noElement;
		m2Input2=noElement;
		output1=noElement;
		output2=noElement;
	}
	~CircularPE() {}
	void readInput() ;
	virtual double sendOutput(int) ;
	void doClockCalculations() ; 	
	void setMode(int inputMode);
	void showInputVariables();
	void showOutputVariables();

//Access functions to residentData/m1Output1 
//These are used only in the case which matrix dimension is 3.
 	void setResidentData(double inputData) {residentData=inputData ;}
	double outputOutput1(){ return output1;}

// Pointer that designates the Above PE of current Circular PE
	CircularPE *  pAbovePE ;
// Pointer that designates the oblique input of current Circular PE :
    SquarePE *  pObliqueInput;

	double output1; // Mode 1 output variable ,also Mode 2 output variable
	double m2Input1 ;// Mode 2 input variable 
	double m1Input1;// Mode 1 input variable 
private:
	// Mode 1 variables  :
	double m1Input2;
	double output2;
	// Mode 2 input variable :
	double m2Input2 ;
};

void CircularPE::readInput()
{
	if ((pAbovePE!=NULL) && (pObliqueInput!=NULL)) //Applicable to inner PEs
	{
		if (mode==1)
		{
			if (residentData!=noElement)
			{
			// Check to see whether the inputs are updated, and if so proceed with getting
			// the inputs and doing the calculations. "outputUpdated" variable is also set.
				if ( (*pInterfaceMatrixElement !=noElement) && (pObliqueInput->outputUpdated==true) )
				{
					m1Input1= *pInterfaceMatrixElement;
					m1Input2= pObliqueInput->sendOutput(NULL) ;

					pObliqueInput->outputNumber=pObliqueInput->outputNumber-1;
					if (pObliqueInput->outputNumber==0)
						pObliqueInput->outputUpdated=false;

					readyForCalculation=true;
				}
				else
				{
					m1Input1=noElement;
					m1Input2=noElement;
				}

			}
			else  // (residentData==noElement) is true
			{
			// Check to see whether the input is updated, and if so proceed with getting
			// the input and setting the residentData
			if (*pInterfaceMatrixElement !=noElement)
				{
					m1Input1= *pInterfaceMatrixElement;
					residentData=m1Input1;
				}
			else
				m1Input1=noElement;
			}
		}
		else // ( 2nd mode )
		{
			if (residentData!=noElement)
			{
			// Check to see whether the inputs are updated, and if so proceed with getting
			// the inputs and doing the calculations. "outputUpdated" variable is also set.

				if ( (pAbovePE->outputUpdated==true) && (pObliqueInput->outputUpdated==true) )
					{
						m2Input1= pAbovePE->sendOutput(BELOW_PE) ;
						m2Input2= pObliqueInput->sendOutput(NULL) ;

					pAbovePE->outputNumber=pAbovePE->outputNumber-1;
					if (pAbovePE->outputNumber==0)
						pAbovePE->outputUpdated=false;
					pObliqueInput->outputNumber=pObliqueInput->outputNumber-1;
					if (pObliqueInput->outputNumber==0)
						pObliqueInput->outputUpdated=false;

						readyForCalculation=true;
					}
				else
				{
					m2Input1=noElement;
					m2Input2=noElement;
				}
			}
			else  // (residentData==noElement) is true
			{
			// Check to see whether the input is updated, and if so proceed with getting
			// the input and setting the residentData
	
				if (pAbovePE->outputUpdated==true)
					{
						m2Input1= pAbovePE->sendOutput(BELOW_PE) ;
						pAbovePE->outputUpdated=false;
						outputUpdated=false;
						residentData=m2Input1;
					} 
				else
					m2Input1=noElement;
			}
		}
	}
	else//Applicable to outer PEs
	{
		if (pAbovePE==NULL) //Applicable to the first PE
		{
		if (mode==1)
		{
			if (residentData!=noElement)
			{
			// Check to see whether the inputs are updated, and if so proceed with getting
			// the inputs and doing the calculations. "outputUpdated" variable is also set.
				if ( (*pInterfaceMatrixElement !=noElement) && (pObliqueInput->outputUpdated==true) )
				{
					m1Input1= *pInterfaceMatrixElement;
					m1Input2= pObliqueInput->sendOutput(NULL) ;

					pObliqueInput->outputNumber=pObliqueInput->outputNumber-1;
					if (pObliqueInput->outputNumber==0)
						pObliqueInput->outputUpdated=false;

					readyForCalculation=true;
				}
				else
				{
					m1Input1=noElement;
					m1Input2=noElement;
				}
			}
			else  // (residentData==noElement) is true
			{
			// Check to see whether the input is updated, and if so proceed with getting
			// the input and setting the residentData
			if (*pInterfaceMatrixElement !=noElement)
				{
					m1Input1= *pInterfaceMatrixElement;
					residentData=m1Input1;
				}
			else
				m1Input1=noElement;
			}
		}
		else // ( 2nd mode )
		{
			outputUpdated=false;

		}
		}
		else // (pObliqueInput==NULL)) is true 
			//Applicable to the last PE
		{
		if (mode==1)
		{
			if (residentData!=noElement)
			{
			// Check to see whether the input is updated, and if so proceed with getting
			// the input and doing the calculations. "outputUpdated" variable is also set.
				if (*pInterfaceMatrixElement !=noElement) 
				{
					m1Input1= *pInterfaceMatrixElement;
					readyForCalculation=true;
				}
				else
					m1Input1=noElement;
			}
			else  // (residentData==noElement) is true
			{
			// Check to see whether the input is updated, and if so proceed with getting
			// the input and setting the residentData
			if (*pInterfaceMatrixElement !=noElement)
				{
					m1Input1= *pInterfaceMatrixElement;
					residentData=m1Input1;
				}
			else
				m1Input1=noElement;
			}
		}
		else // ( 2nd mode )
		{
		// Check to see whether the input is updated, and if so proceed with getting
		// the input and setting the residentData
			if (pAbovePE->outputUpdated==true)
				{
					m2Input1= pAbovePE->sendOutput(BELOW_PE) ;

					pAbovePE->outputNumber=pAbovePE->outputNumber-1;
					if (pAbovePE->outputNumber==0)
						pAbovePE->outputUpdated=false;

					outputUpdated=false;
					residentData=m2Input1;
				}
			else
				m2Input1=noElement;
		}
		}
	}
}

double CircularPE::sendOutput(int outputDirection )
{
	if (mode==1)
	{
		if(outputDirection==BELOW_PE)
			return output1 ;
		if(outputDirection==LEFT_PE)
			return output2;
	}
	else 
	{
		if(outputDirection==BELOW_PE)
			return output1;
	}
}

void CircularPE::doClockCalculations()
{
	if (readyForCalculation==true)
	{
		if (mode==1)
		{
			output1=m1Input1-(m1Input2* residentData);
			output2=residentData;
			outputUpdated=true;
			readyForCalculation=false;
		}
		else
		{
			output1=m2Input1-(m2Input2* residentData);
			outputUpdated=true;
			readyForCalculation=false;
		}
	}
}
void CircularPE::setMode(int inputMode)
	{
		mode=inputMode;
		if (mode==1)
			outputNumber=2;
		else
			outputNumber=1;
	}

void CircularPE::showInputVariables()
	{
		if (mode==1)
		{
			if (m1Input1!=noElement)
				cout<<" *Input 1:"<<m1Input1 ;
			else
				cout<<" *Input 1:"<<"No Element" ;
			if (m1Input2!=noElement)
				cout<<" *Input 2:"<<m1Input2 ;
			else
				cout<<" *Input 2:"<<"No Element" ;
		}
		else
		{
			if (m2Input1!=noElement)
				cout<<" *Input 1:"<<m2Input1 ;
			else
				cout<<" *Input 1:"<<"No Element" ;
			if (m2Input2!=noElement)
				cout<<" *Input 2:"<<m2Input2 ;
			else
				cout<<" *Input 2:"<<"No Element" ;
		}
	}

void CircularPE::showOutputVariables()
	{
		if (mode==1)
		{
			if (output1!=noElement)
				cout<<" *Output 1:"<<output1 ;
			else
				cout<<" *Output 1:"<<"No Element" ;
			if (output2!=noElement)
				cout<<" *Output 2:"<<output2 ;
			else
				cout<<" *Output 2:"<<"No Element" ;
		}
		else
		{
			if (output1!=noElement)
				cout<<" *Output :"<<output1 ;
			else
				cout<<" *Output :"<<"No Element" ;
		}
	}

void main()
{
	int i,j ;
	char reenter,restart;
	double tempMatrix[matrixDimension][matrixDimension];
	double TM[matrixDimension][matrixDimension];
    double matrix[matrixDimension]; //The Formatted interface matrix in each clock
	double unity[matrixDimension];	//The Formatted interface unity matrix in each clock
	double invertedMatrix[matrixDimension][matrixDimension];
	double detTM,detTM1,detTM2,detTM3;
	
	cout<<"********************************************************************************"<<endl;
	cout<<"*************** This program is a simulator for the CSA approach ***************" <<endl;
	cout<<"***************        in calculating the matrix inversion.      ***************" << endl;
	cout<<"********************************************************************************"<<endl;
	cout<<"In this program, although great bulk of the code has been implemented "<<endl;
	cout<<"regardless of the actual matrix dimension, due to some algorithm development "<<endl ;
	cout<<"difficulties, matrix dimension can only be 3."<< endl ;
restartingTheProgram:
reenteringTheMatrix:
	cout<<"For entering the matrix, please use this format:"<< endl ;
	cout<<"(Typing each element and then pressing the Enter key) :"<< endl ;
    for (i=0; i<matrixDimension; i++ )
		for (j=0; j<matrixDimension; j++ )
		{
			cin>> tempMatrix[i][j];
		}
	cout<< "This is the entered matrix:"<< endl ;
    for (i=0; i<matrixDimension; i++ )
	{
		for (j=0; j<matrixDimension; j++ )
		{
			cout.width( 15 );
			cout<< setiosflags( ios::left )<< setiosflags( ios::fixed )<<setprecision(4)<<tempMatrix[i][j];
		}
		cout<< endl;
	}
	cout<<"Do you want to reenter the matrix?(Y/N)"<<endl;        
	cin>> reenter;
	if ((reenter=='y') || (reenter=='Y') )
		goto reenteringTheMatrix;
	cin.ignore();
	//Checking to see whether the determinant is zero
		for(i=0; i<3; i++)
			for(j=0; j<3; j++)
				TM[i][j]=tempMatrix[i][j];	

			detTM1=TM[0][0] * (TM[1][1]*TM[2][2]- TM[1][2]* TM[2][1]);
			detTM2=TM[0][1] * (TM[1][0]*TM[2][2]- TM[1][2]* TM[2][0]);
			detTM3=TM[0][2] * (TM[1][0]*TM[2][1]- TM[1][1]* TM[2][0]);
			detTM=detTM1-detTM2+detTM3;
	if (detTM==0)
	{
		cout<<endl <<"The determinant is zero, matrix inversion can not be performed.";
		cout<<endl << "Please enter another matrix." << endl;
		goto reenteringTheMatrix ;
	}
// Constructing the unity matrix exclusively for the 3 matrix dimension.
	double tempUnity[3][7]={
		{noElement,1,0,0,noElement,noElement,noElement},
		{noElement,noElement,noElement,noElement,noElement,noElement,0},
		{noElement,noElement,noElement,noElement,noElement,noElement,noElement}} ;
//Constructing the CSA in the memory:
	//Creating array of pointers to objects in the stack:
	SquarePE * pSquarePEGrid[matrixDimension];
	CircularPE * pCircularPEGrid[matrixDimension];
    EllipticPE  * pEllipticPEGrid[matrixDimension-1][matrixDimension-1];
	//Initializing the created array of pointers to objects to allocated memory in the freestore : 
	for (i=0 ; i < matrixDimension ; i++)
	{
	pSquarePEGrid[i] = new SquarePE;
	pCircularPEGrid[i] = new CircularPE;
	}

    for (i=0 ; i < matrixDimension-1 ; i++)
		for (j=0 ; j < matrixDimension-1 ; j++)
		{
			//Allocating memory for the upper triangle
			if (j>=i)
				pEllipticPEGrid[i][j]= new EllipticPE;
			else
				//Skipping the memory allocation for the lower triangle
				pEllipticPEGrid[i][j]=0;
		}
	//Initializing the pointers that designate the Above/Below/Right/Left PE for each Elliptic PE :
	  //Setting the inner PEs :
      for (i=1 ; i < matrixDimension-2 ; i++)
		for (int j=1 ; j < matrixDimension-2 ; j++)
		{
			if (j>i)   //Upper triangle
			{
				pEllipticPEGrid[i][j] ->pAbovePE = pEllipticPEGrid[i-1][j] ;
				pEllipticPEGrid[i][j] ->pBelowPE = pEllipticPEGrid[i+1][j] ;
				pEllipticPEGrid[i][j] ->pLeftPE = pEllipticPEGrid[i][j-1] ;
                pEllipticPEGrid[i][j] ->pRightPE = pEllipticPEGrid[i][j+1] ;
			}
		}
	  //Setting the first row except PEs at both ends :
		for (j=1 ; j < matrixDimension-2 ; j++)
		{
				pEllipticPEGrid[0][j] ->pAbovePE = 0 ;
				pEllipticPEGrid[0][j] ->pBelowPE = pEllipticPEGrid[1][j] ;
				pEllipticPEGrid[0][j] ->pLeftPE = pEllipticPEGrid[0][j-1] ;
                pEllipticPEGrid[0][j] ->pRightPE = pEllipticPEGrid[0][j+1] ;
		}
	  //Setting the last column except PEs at both ends :	
		for (i=1 ; i < matrixDimension-2 ; i++)
		{
				pEllipticPEGrid[i][matrixDimension-2] ->pAbovePE = pEllipticPEGrid[i+1][matrixDimension-2] ;
				pEllipticPEGrid[i][matrixDimension-2] ->pBelowPE = pEllipticPEGrid[i-1][matrixDimension-2] ;
				pEllipticPEGrid[i][matrixDimension-2] ->pLeftPE  = pEllipticPEGrid[i][matrixDimension-2-1] ;
                pEllipticPEGrid[i][matrixDimension-2] ->pRightPE = pCircularPEGrid[i] ;
				pEllipticPEGrid[i][matrixDimension-2] ->setRightEndOfTheGrid(true);
		}
	  //Setting the Diameter except PEs at both ends :
		for (i=1 ; i < matrixDimension-2 ; i++)
		{
				pEllipticPEGrid[i][i] ->pAbovePE = pEllipticPEGrid[i+1][i] ;
                pEllipticPEGrid[i][i] ->pRightPE = pEllipticPEGrid[i][i+1] ;
				pEllipticPEGrid[i][i] ->pBelowPE = pSquarePEGrid[i+1] ;
				pEllipticPEGrid[i][i] ->pLeftPE = pSquarePEGrid[i] ;
		}
      //Setting the corner PEs of the triangle :
/*		switch (matrixDimension )
		{

		case 1: 
			cout<<"The dimension is one ! There is no need for this program buddy !" ;
			break ;

		case 2:
			{
			pEllipticPEGrid[0][0] ->pAbovePE = 0 ;
			pEllipticPEGrid[0][0] ->pRightPE = pCircularPEGrid[0] ;
			pEllipticPEGrid[0][0] ->pBelowPE = pSquarePEGrid[1] ;
			pEllipticPEGrid[0][0] ->pLeftPE = pSquarePEGrid[0] ;
			};
			break ;
		default :
*/		{
				pEllipticPEGrid[0][0] ->pAbovePE = 0 ;
                pEllipticPEGrid[0][0] ->pRightPE = pEllipticPEGrid[0][1] ;
				pEllipticPEGrid[0][0] ->pBelowPE = pSquarePEGrid[1] ;
				pEllipticPEGrid[0][0] ->pLeftPE = pSquarePEGrid[0] ;

				pEllipticPEGrid[0][matrixDimension-2] ->pAbovePE = 0 ;
                pEllipticPEGrid[0][matrixDimension-2] ->pRightPE = pCircularPEGrid[0] ;
				pEllipticPEGrid[0][matrixDimension-2] ->pBelowPE = pEllipticPEGrid[1][matrixDimension-2] ;
				pEllipticPEGrid[0][matrixDimension-2] ->pLeftPE = pEllipticPEGrid[0][matrixDimension-2-1] ;
				pEllipticPEGrid[0][matrixDimension-2] ->setRightEndOfTheGrid(true);

				pEllipticPEGrid[matrixDimension-2][matrixDimension-2] ->pAbovePE = pEllipticPEGrid[matrixDimension-2-1][matrixDimension-2] ;
                pEllipticPEGrid[matrixDimension-2][matrixDimension-2] ->pRightPE = pCircularPEGrid[matrixDimension-2] ;
				pEllipticPEGrid[matrixDimension-2][matrixDimension-2] ->pBelowPE = pSquarePEGrid[matrixDimension-1] ;
				pEllipticPEGrid[matrixDimension-2][matrixDimension-2] ->pLeftPE = pSquarePEGrid[matrixDimension-2] ;
				pEllipticPEGrid[matrixDimension-2][matrixDimension-2] ->setRightEndOfTheGrid(true);
		}
//		}

	//Initializing the pointers that designate the Above/Right PE in each Square PE :
	  //Setting the inner PEs :
		for (i=1 ; i < matrixDimension-1 ; i++)
		{
				pSquarePEGrid[i] ->pAbovePE = pEllipticPEGrid[i-1][i-1] ;
                pSquarePEGrid[i] ->pRightPE = pEllipticPEGrid[i][i] ;
		}

	  //Setting the PEs at both ends :
				pSquarePEGrid[0] ->pAbovePE = 0 ;
                pSquarePEGrid[0] ->pRightPE = pEllipticPEGrid[0][0] ;

				pSquarePEGrid[matrixDimension-1] ->pAbovePE = pEllipticPEGrid[matrixDimension-2][matrixDimension-2] ;
                pSquarePEGrid[matrixDimension-1] ->pRightPE = pCircularPEGrid[matrixDimension-1] ;
	//Initializing the pointers that designate the Above PE & oblique input in each Circular PE :
	  //Setting the above PEs except for the first PE and oblique inputs except for the last PE
		for (i=1 ; i < matrixDimension ; i++)
		{
				pCircularPEGrid[i] ->pAbovePE = pCircularPEGrid[i-1] ;
                pCircularPEGrid[i-1] ->pObliqueInput = pSquarePEGrid[i-1] ;
		}
      //Setting the above PE of the first PE and oblique input of the last PE
				pCircularPEGrid[0] ->pAbovePE = 0;
                pCircularPEGrid[matrixDimension-1] ->pObliqueInput = 0 ;
//Initializing "pInterfaceMatrixElement" elements which designate the interface matrix elements. 
//exclusively for the case that the matrix dimension is 3 :
	pEllipticPEGrid[0][0] ->pInterfaceMatrixElement = &matrix[1];
	pEllipticPEGrid[0][matrixDimension-2] ->pInterfaceMatrixElement = &matrix[2];
	pSquarePEGrid[0] ->pInterfaceMatrixElement = &matrix[0];
	for (i=0 ; i < matrixDimension ; i++)
		pCircularPEGrid[i] ->pInterfaceMatrixElement = &unity[i];

// Clock Calculations exclusively for the case that the matrix dimension is 3 :
 for (int clock=1 ; clock<=15 ; clock++ )
 {
	switch (clock)
	{
		case 1:
			//Setting the PE modes:
			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->setMode(1);
				pCircularPEGrid[i]->setMode(1);
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->setMode(1);
			//Setting the interface matrices from input matrix and customized unity matrix:
			matrix[0]=tempMatrix[0][0];
			matrix[1]=noElement;
			matrix[2]=noElement;
			unity[0]=tempUnity[0][0];
			unity[1]=tempUnity[1][0];
			unity[2]=tempUnity[2][0];
			//Looping through all the PEs and doing the clock calculations:
			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->readInput();
				pCircularPEGrid[i]->readInput();
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->readInput();

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->doClockCalculations();
				pCircularPEGrid[i]->doClockCalculations();
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->doClockCalculations();

	cout<<"*******************************************************************************"<<endl;
	cout<<"Clock 1:" <<endl;
	cout<<"*******************************************************************************"<<endl;
	cout<<"PE 1:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[0]->showMode() ;
	pSquarePEGrid[0]->showInputVariables() ;
	pSquarePEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 2:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][0]->showMode() ;
	pEllipticPEGrid[0][0]->showInputVariables() ;
	pEllipticPEGrid[0][0]->outputResidentData() ;
	cout<<endl <<"      ";
	pEllipticPEGrid[0][0]->showOutputVariables();

	cout<<endl<<"PE 3:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][1]->showMode() ;
	pEllipticPEGrid[0][1]->showInputVariables() ;
	pEllipticPEGrid[0][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[0][1]->showOutputVariables();

	cout<<endl<<"PE 4:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[1]->showMode() ;
	pSquarePEGrid[1]->showInputVariables() ;
	pSquarePEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 5:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[1][1]->showMode() ;
	pEllipticPEGrid[1][1]->showInputVariables() ;
	pEllipticPEGrid[1][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[1][1]->showOutputVariables();

	cout<<endl<<"PE 6:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[2]->showMode() ;
	pSquarePEGrid[2]->showInputVariables() ;
	pSquarePEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[2]->showOutputVariables();

	cout<<endl<<"PE 7:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[0]->showMode() ;
	pCircularPEGrid[0]->showInputVariables() ;
	pCircularPEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 8:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[1]->showMode() ;
	pCircularPEGrid[1]->showInputVariables() ;
	pCircularPEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 9:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[2]->showMode() ;
	pCircularPEGrid[2]->showInputVariables() ;
	pCircularPEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[2]->showOutputVariables();

    cout<<endl <<endl<<"Press the Enter key to see the next clock values...";
	cin.get();	
			break;			

		case 2:
			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->setMode(1);
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->setMode(1);

			pCircularPEGrid[0]->setMode(1);
			pCircularPEGrid[1]->setMode(2);
			pCircularPEGrid[2]->setMode(2);

			matrix[0]=tempMatrix[1][0];
			matrix[1]=tempMatrix[0][1];
			matrix[2]=noElement;
			unity[0]=tempUnity[0][1];
			unity[1]=tempUnity[1][1];
			unity[2]=tempUnity[2][1];

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->readInput();
				pCircularPEGrid[i]->readInput();
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->readInput();

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->doClockCalculations();
				pCircularPEGrid[i]->doClockCalculations();
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->doClockCalculations();
	cout<<"*******************************************************************************"<<endl;
	cout<<"Clock 2:" <<endl;
	cout<<"*******************************************************************************"<<endl;
	cout<<"PE 1:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[0]->showMode() ;
	pSquarePEGrid[0]->showInputVariables() ;
	pSquarePEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 2:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][0]->showMode() ;
	pEllipticPEGrid[0][0]->showInputVariables() ;
	pEllipticPEGrid[0][0]->outputResidentData() ;
	cout<<endl <<"      ";
	pEllipticPEGrid[0][0]->showOutputVariables();

	cout<<endl<<"PE 3:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][1]->showMode() ;
	pEllipticPEGrid[0][1]->showInputVariables() ;
	pEllipticPEGrid[0][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[0][1]->showOutputVariables();

	cout<<endl<<"PE 4:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[1]->showMode() ;
	pSquarePEGrid[1]->showInputVariables() ;
	pSquarePEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 5:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[1][1]->showMode() ;
	pEllipticPEGrid[1][1]->showInputVariables() ;
	pEllipticPEGrid[1][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[1][1]->showOutputVariables();

	cout<<endl<<"PE 6:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[2]->showMode() ;
	pSquarePEGrid[2]->showInputVariables() ;
	pSquarePEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[2]->showOutputVariables();

	cout<<endl<<"PE 7:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[0]->showMode() ;
	pCircularPEGrid[0]->showInputVariables() ;
	pCircularPEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 8:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[1]->showMode() ;
	pCircularPEGrid[1]->showInputVariables() ;
	pCircularPEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 9:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[2]->showMode() ;
	pCircularPEGrid[2]->showInputVariables() ;
	pCircularPEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[2]->showOutputVariables();

    cout<<endl <<endl<<"Press the Enter key to see the next clock values...";
	cin.get();	
			break;

		case 3:
			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->setMode(1);
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->setMode(1);

			pCircularPEGrid[0]->setMode(1);
			pCircularPEGrid[1]->setMode(2);
			pCircularPEGrid[2]->setMode(2);

			matrix[0]=tempMatrix[2][0];
			matrix[1]=tempMatrix[1][1];
			matrix[2]=tempMatrix[0][2];
			unity[0]=tempUnity[0][2];
			unity[1]=tempUnity[1][2];
			unity[2]=tempUnity[2][2];

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->readInput();
				pCircularPEGrid[i]->readInput();
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->readInput();

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->doClockCalculations();	
				pCircularPEGrid[i]->doClockCalculations();	
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->doClockCalculations();
	cout<<"*******************************************************************************"<<endl;
	cout<<"Clock 3:" <<endl;
	cout<<"*******************************************************************************"<<endl;
	cout<<"PE 1:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[0]->showMode() ;
	pSquarePEGrid[0]->showInputVariables() ;
	pSquarePEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 2:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][0]->showMode() ;
	pEllipticPEGrid[0][0]->showInputVariables() ;
	pEllipticPEGrid[0][0]->outputResidentData() ;
	cout<<endl <<"      ";
	pEllipticPEGrid[0][0]->showOutputVariables();

	cout<<endl<<"PE 3:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][1]->showMode() ;
	pEllipticPEGrid[0][1]->showInputVariables() ;
	pEllipticPEGrid[0][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[0][1]->showOutputVariables();

	cout<<endl<<"PE 4:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[1]->showMode() ;
	pSquarePEGrid[1]->showInputVariables() ;
	pSquarePEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 5:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[1][1]->showMode() ;
	pEllipticPEGrid[1][1]->showInputVariables() ;
	pEllipticPEGrid[1][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[1][1]->showOutputVariables();

	cout<<endl<<"PE 6:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[2]->showMode() ;
	pSquarePEGrid[2]->showInputVariables() ;
	pSquarePEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[2]->showOutputVariables();

	cout<<endl<<"PE 7:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[0]->showMode() ;
	pCircularPEGrid[0]->showInputVariables() ;
	pCircularPEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 8:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[1]->showMode() ;
	pCircularPEGrid[1]->showInputVariables() ;
	pCircularPEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 9:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[2]->showMode() ;
	pCircularPEGrid[2]->showInputVariables() ;
	pCircularPEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[2]->showOutputVariables();

    cout<<endl <<endl<<"Press the Enter key to see the next clock values...";
	cin.get();	
			break;
		case 4:
			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->setMode(1);
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->setMode(1);

			pCircularPEGrid[0]->setMode(1);
			pCircularPEGrid[1]->setMode(2);
			pCircularPEGrid[2]->setMode(2);
			
			matrix[0]=noElement;
			matrix[1]=tempMatrix[2][1];
			matrix[2]=tempMatrix[1][2];
			unity[0]=tempUnity[0][3];
			unity[1]=tempUnity[1][3];
			unity[2]=tempUnity[2][3];

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->readInput();
				pCircularPEGrid[i]->readInput();
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->readInput();

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->doClockCalculations();	
				pCircularPEGrid[i]->doClockCalculations();	
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->doClockCalculations();
	cout<<"*******************************************************************************"<<endl;
	cout<<"Clock 4:" <<endl;
	cout<<"*******************************************************************************"<<endl;
	cout<<"PE 1:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[0]->showMode() ;
	pSquarePEGrid[0]->showInputVariables() ;
	pSquarePEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 2:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][0]->showMode() ;
	pEllipticPEGrid[0][0]->showInputVariables() ;
	pEllipticPEGrid[0][0]->outputResidentData() ;
	cout<<endl <<"      ";
	pEllipticPEGrid[0][0]->showOutputVariables();

	cout<<endl<<"PE 3:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][1]->showMode() ;
	pEllipticPEGrid[0][1]->showInputVariables() ;
	pEllipticPEGrid[0][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[0][1]->showOutputVariables();

	cout<<endl<<"PE 4:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[1]->showMode() ;
	pSquarePEGrid[1]->showInputVariables() ;
	pSquarePEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 5:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[1][1]->showMode() ;
	pEllipticPEGrid[1][1]->showInputVariables() ;
	pEllipticPEGrid[1][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[1][1]->showOutputVariables();

	cout<<endl<<"PE 6:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[2]->showMode() ;
	pSquarePEGrid[2]->showInputVariables() ;
	pSquarePEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[2]->showOutputVariables();

	cout<<endl<<"PE 7:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[0]->showMode() ;
	pCircularPEGrid[0]->showInputVariables() ;
	pCircularPEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 8:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[1]->showMode() ;
	pCircularPEGrid[1]->showInputVariables() ;
	pCircularPEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 9:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[2]->showMode() ;
	pCircularPEGrid[2]->showInputVariables() ;
	pCircularPEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[2]->showOutputVariables();

    cout<<endl <<endl<<"Press the Enter key to see the next clock values...";
	cin.get();	

			break;
		case 5:
			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->setMode(1);
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->setMode(1);

			pCircularPEGrid[0]->setMode(1);
			pCircularPEGrid[1]->setMode(1);
			pCircularPEGrid[2]->setMode(2);

			matrix[0]=noElement;
			matrix[1]=noElement;
			matrix[2]=tempMatrix[2][2];
			unity[0]=tempUnity[0][4];
			unity[1]=tempUnity[1][4];
			unity[2]=tempUnity[2][4];

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->readInput();
				pCircularPEGrid[i]->readInput();
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->readInput();

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->doClockCalculations();	
				pCircularPEGrid[i]->doClockCalculations();	
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->doClockCalculations();	
	cout<<"*******************************************************************************"<<endl;
	cout<<"Clock 5:" <<endl;
	cout<<"*******************************************************************************"<<endl;
	cout<<"PE 1:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[0]->showMode() ;
	pSquarePEGrid[0]->showInputVariables() ;
	pSquarePEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 2:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][0]->showMode() ;
	pEllipticPEGrid[0][0]->showInputVariables() ;
	pEllipticPEGrid[0][0]->outputResidentData() ;
	cout<<endl <<"      ";
	pEllipticPEGrid[0][0]->showOutputVariables();

	cout<<endl<<"PE 3:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][1]->showMode() ;
	pEllipticPEGrid[0][1]->showInputVariables() ;
	pEllipticPEGrid[0][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[0][1]->showOutputVariables();

	cout<<endl<<"PE 4:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[1]->showMode() ;
	pSquarePEGrid[1]->showInputVariables() ;
	pSquarePEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 5:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[1][1]->showMode() ;
	pEllipticPEGrid[1][1]->showInputVariables() ;
	pEllipticPEGrid[1][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[1][1]->showOutputVariables();

	cout<<endl<<"PE 6:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[2]->showMode() ;
	pSquarePEGrid[2]->showInputVariables() ;
	pSquarePEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[2]->showOutputVariables();

	cout<<endl<<"PE 7:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[0]->showMode() ;
	pCircularPEGrid[0]->showInputVariables() ;
	pCircularPEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 8:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[1]->showMode() ;
	pCircularPEGrid[1]->showInputVariables() ;
	pCircularPEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 9:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[2]->showMode() ;
	pCircularPEGrid[2]->showInputVariables() ;
	pCircularPEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[2]->showOutputVariables();

    cout<<endl <<endl<<"Press the Enter key to see the next clock values...";
	cin.get();	
			break;
		case 6:
			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->setMode(1);
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->setMode(1);

			pCircularPEGrid[0]->setMode(1);
			pCircularPEGrid[1]->setMode(2);
			pCircularPEGrid[2]->setMode(2);

			matrix[0]=noElement;
			matrix[1]=noElement;
			matrix[2]=noElement;
			unity[0]=tempUnity[0][5];
			unity[1]=tempUnity[1][5];
			unity[2]=tempUnity[2][5];

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->readInput();
				pCircularPEGrid[i]->readInput();
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->readInput();

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->doClockCalculations();	
				pCircularPEGrid[i]->doClockCalculations();	
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->doClockCalculations();	
	cout<<"*******************************************************************************"<<endl;
	cout<<"Clock 6:" <<endl;
	cout<<"*******************************************************************************"<<endl;
	cout<<"PE 1:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[0]->showMode() ;
	pSquarePEGrid[0]->showInputVariables() ;
	pSquarePEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 2:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][0]->showMode() ;
	pEllipticPEGrid[0][0]->showInputVariables() ;
	pEllipticPEGrid[0][0]->outputResidentData() ;
	cout<<endl <<"      ";
	pEllipticPEGrid[0][0]->showOutputVariables();

	cout<<endl<<"PE 3:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][1]->showMode() ;
	pEllipticPEGrid[0][1]->showInputVariables() ;
	pEllipticPEGrid[0][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[0][1]->showOutputVariables();

	cout<<endl<<"PE 4:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[1]->showMode() ;
	pSquarePEGrid[1]->showInputVariables() ;
	pSquarePEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 5:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[1][1]->showMode() ;
	pEllipticPEGrid[1][1]->showInputVariables() ;
	pEllipticPEGrid[1][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[1][1]->showOutputVariables();

	cout<<endl<<"PE 6:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[2]->showMode() ;
	pSquarePEGrid[2]->showInputVariables() ;
	pSquarePEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[2]->showOutputVariables();

	cout<<endl<<"PE 7:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[0]->showMode() ;
	pCircularPEGrid[0]->showInputVariables() ;
	pCircularPEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 8:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[1]->showMode() ;
	pCircularPEGrid[1]->showInputVariables() ;
	pCircularPEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 9:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[2]->showMode() ;
	pCircularPEGrid[2]->showInputVariables() ;
	pCircularPEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[2]->showOutputVariables();

    cout<<endl <<endl<<"Press the Enter key to see the next clock values...";
	cin.get();	

			break;
		case 7:
			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->setMode(1);
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->setMode(1);

			pCircularPEGrid[0]->setMode(2);
			pCircularPEGrid[1]->setMode(1);
			pCircularPEGrid[2]->setMode(2);

			matrix[0]=noElement;
			matrix[1]=noElement;
			matrix[2]=noElement;
			unity[0]=tempUnity[0][6];
			unity[1]=tempUnity[1][6];
			unity[2]=tempUnity[2][6];

			pSquarePEGrid[1]->outputUpdated=true ;

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->readInput();
				pCircularPEGrid[i]->readInput();
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->readInput();

			pEllipticPEGrid[1][1]->readyForCalculation=false;
			
			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->doClockCalculations();	
				pCircularPEGrid[i]->doClockCalculations();	
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->doClockCalculations();	

			pCircularPEGrid[1]->output1=(pCircularPEGrid[1]->output1)/(pCircularPEGrid[1]->showResidentData());
	cout<<"*******************************************************************************"<<endl;
	cout<<"Clock 7:" <<endl;
	cout<<"*******************************************************************************"<<endl;
	cout<<"PE 1:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[0]->showMode() ;
	pSquarePEGrid[0]->showInputVariables() ;
	pSquarePEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 2:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][0]->showMode() ;
	pEllipticPEGrid[0][0]->showInputVariables() ;
	pEllipticPEGrid[0][0]->outputResidentData() ;
	cout<<endl <<"      ";
	pEllipticPEGrid[0][0]->showOutputVariables();

	cout<<endl<<"PE 3:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][1]->showMode() ;
	pEllipticPEGrid[0][1]->showInputVariables() ;
	pEllipticPEGrid[0][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[0][1]->showOutputVariables();

	cout<<endl<<"PE 4:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[1]->showMode() ;
	pSquarePEGrid[1]->showInputVariables() ;
	pSquarePEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 5:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[1][1]->showMode() ;
	pEllipticPEGrid[1][1]->showInputVariables() ;
	pEllipticPEGrid[1][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[1][1]->showOutputVariables();

	cout<<endl<<"PE 6:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[2]->showMode() ;
	pSquarePEGrid[2]->showInputVariables() ;
	pSquarePEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[2]->showOutputVariables();

	cout<<endl<<"PE 7:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[0]->showMode() ;
	pCircularPEGrid[0]->showInputVariables() ;
	pCircularPEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 8:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[1]->showMode() ;
	pCircularPEGrid[1]->showInputVariables() ;
	pCircularPEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 9:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[2]->showMode() ;
	pCircularPEGrid[2]->showInputVariables() ;
	pCircularPEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[2]->showOutputVariables();

    cout<<endl <<endl<<"Press the Enter key to see the next clock values...";
	cin.get();	

			break;
		case 8:
			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->setMode(1);
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->setMode(1);

			pCircularPEGrid[0]->setMode(2);
			pCircularPEGrid[1]->setMode(2);
			pCircularPEGrid[2]->setMode(1);

			matrix[0]=noElement;
			matrix[1]=noElement;
			matrix[2]=noElement;
			unity[0]=noElement;
			unity[1]=noElement;
			unity[2]=noElement;

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->readInput();
				pCircularPEGrid[i]->readInput();
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->readInput();

			pCircularPEGrid[2]->readyForCalculation=true;

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->doClockCalculations();	
				pCircularPEGrid[i]->doClockCalculations();	
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->doClockCalculations();	
	cout<<"*******************************************************************************"<<endl;
	cout<<"Clock 8:" <<endl;
	cout<<"*******************************************************************************"<<endl;
	cout<<"PE 1:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[0]->showMode() ;
	pSquarePEGrid[0]->showInputVariables() ;
	pSquarePEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 2:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][0]->showMode() ;
	pEllipticPEGrid[0][0]->showInputVariables() ;
	pEllipticPEGrid[0][0]->outputResidentData() ;
	cout<<endl <<"      ";
	pEllipticPEGrid[0][0]->showOutputVariables();

	cout<<endl<<"PE 3:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][1]->showMode() ;
	pEllipticPEGrid[0][1]->showInputVariables() ;
	pEllipticPEGrid[0][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[0][1]->showOutputVariables();

	cout<<endl<<"PE 4:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[1]->showMode() ;
	pSquarePEGrid[1]->showInputVariables() ;
	pSquarePEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 5:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[1][1]->showMode() ;
	pEllipticPEGrid[1][1]->showInputVariables() ;
	pEllipticPEGrid[1][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[1][1]->showOutputVariables();

	cout<<endl<<"PE 6:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[2]->showMode() ;
	pSquarePEGrid[2]->showInputVariables() ;
	pSquarePEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[2]->showOutputVariables();

	cout<<endl<<"PE 7:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[0]->showMode() ;
	pCircularPEGrid[0]->showInputVariables() ;
	pCircularPEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 8:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[1]->showMode() ;
	pCircularPEGrid[1]->showInputVariables() ;
	pCircularPEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 9:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[2]->showMode() ;
	pCircularPEGrid[2]->showInputVariables() ;
	pCircularPEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[2]->showOutputVariables();

	//Second half clock :
	pCircularPEGrid[2]->setMode(2);
	pCircularPEGrid[2]->setResidentData( pCircularPEGrid[1]->outputOutput1() );
	pCircularPEGrid[2]->m2Input1=pCircularPEGrid[1]->outputOutput1();

	cout<<endl<<"PE 9 (Second half clock):" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[2]->showMode() ;
	pCircularPEGrid[2]->showInputVariables() ;
	pCircularPEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[2]->showOutputVariables();

    cout<<endl <<endl<<"Press the Enter key to see the next clock values...";
	cin.get();	

			break;
		case 9:
			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->setMode(2);
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->setMode(2);

			pCircularPEGrid[0]->setMode(1);
			pCircularPEGrid[1]->setMode(1);
			pCircularPEGrid[2]->setMode(1);

			matrix[0]=noElement;
			matrix[1]=noElement;
			matrix[2]=noElement;
			unity[0]=noElement;
			unity[1]=noElement;
			unity[2]=noElement;

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->readInput();
				pCircularPEGrid[i]->readInput();
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->readInput();

			pCircularPEGrid[1]->readyForCalculation=true;
			pCircularPEGrid[2]->readyForCalculation=true;

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->doClockCalculations();	
				pCircularPEGrid[i]->doClockCalculations();	
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->doClockCalculations();	

	cout<<"*******************************************************************************"<<endl;
	cout<<"Clock 9:" <<endl;
	cout<<"*******************************************************************************"<<endl;
	cout<<"PE 1:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[0]->showMode() ;
	pSquarePEGrid[0]->showInputVariables() ;
	pSquarePEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 2:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][0]->showMode() ;
	pEllipticPEGrid[0][0]->showInputVariables() ;
	pEllipticPEGrid[0][0]->outputResidentData() ;
	cout<<endl <<"      ";
	pEllipticPEGrid[0][0]->showOutputVariables();

	cout<<endl<<"PE 3:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][1]->showMode() ;
	pEllipticPEGrid[0][1]->showInputVariables() ;
	pEllipticPEGrid[0][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[0][1]->showOutputVariables();

	cout<<endl<<"PE 4:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[1]->showMode() ;
	pSquarePEGrid[1]->showInputVariables() ;
	pSquarePEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 5:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[1][1]->showMode() ;
	pEllipticPEGrid[1][1]->showInputVariables() ;
	pEllipticPEGrid[1][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[1][1]->showOutputVariables();

	cout<<endl<<"PE 6:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[2]->showMode() ;
	pSquarePEGrid[2]->showInputVariables() ;
	pSquarePEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[2]->showOutputVariables();

	cout<<endl<<"PE 7:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[0]->showMode() ;
	pCircularPEGrid[0]->showInputVariables() ;
	pCircularPEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 8:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[1]->showMode() ;
	pCircularPEGrid[1]->showInputVariables() ;
	pCircularPEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 9:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[2]->showMode() ;
	pCircularPEGrid[2]->showInputVariables() ;
	pCircularPEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[2]->showOutputVariables();

	//Second half clock :
	pCircularPEGrid[2]->setResidentData( 1 );
	pCircularPEGrid[1]->setResidentData( 1 );
	pCircularPEGrid[2]->m1Input1=noElement;
	pCircularPEGrid[1]->m1Input1=noElement;

	cout<<endl<<"PE 8 (Second half clock):" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[1]->showMode() ;
	pCircularPEGrid[1]->showInputVariables() ;
	pCircularPEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 9 (Second half clock):" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[2]->showMode() ;
	pCircularPEGrid[2]->showInputVariables() ;
	pCircularPEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[2]->showOutputVariables();

    cout<<endl <<endl<<"Press the Enter key to see the next clock values...";
	cin.get();	

			break;
		case 10:
			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->setMode(2);
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->setMode(2);

			pCircularPEGrid[0]->setMode(1);
			pCircularPEGrid[1]->setMode(1);
			pCircularPEGrid[2]->setMode(1);

			matrix[0]=noElement;
			matrix[1]=noElement;
			matrix[2]=noElement;
			unity[0]=noElement;
			unity[1]=noElement;
			unity[2]=noElement;

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->readInput();
				pCircularPEGrid[i]->readInput();
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->readInput();

			pCircularPEGrid[0]->readyForCalculation=true;
			pCircularPEGrid[1]->readyForCalculation=true;
			pCircularPEGrid[2]->readyForCalculation=true;

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->doClockCalculations();	
				pCircularPEGrid[i]->doClockCalculations();	
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->doClockCalculations();	
	cout<<"*******************************************************************************"<<endl;
	cout<<"Clock 10:" <<endl;
	cout<<"*******************************************************************************"<<endl;
	cout<<"PE 1:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[0]->showMode() ;
	pSquarePEGrid[0]->showInputVariables() ;
	pSquarePEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 2:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][0]->showMode() ;
	pEllipticPEGrid[0][0]->showInputVariables() ;
	pEllipticPEGrid[0][0]->outputResidentData() ;
	cout<<endl <<"      ";
	pEllipticPEGrid[0][0]->showOutputVariables();

	cout<<endl<<"PE 3:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][1]->showMode() ;
	pEllipticPEGrid[0][1]->showInputVariables() ;
	pEllipticPEGrid[0][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[0][1]->showOutputVariables();

	cout<<endl<<"PE 4:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[1]->showMode() ;
	pSquarePEGrid[1]->showInputVariables() ;
	pSquarePEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 5:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[1][1]->showMode() ;
	pEllipticPEGrid[1][1]->showInputVariables() ;
	pEllipticPEGrid[1][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[1][1]->showOutputVariables();

	cout<<endl<<"PE 6:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[2]->showMode() ;
	pSquarePEGrid[2]->showInputVariables() ;
	pSquarePEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[2]->showOutputVariables();

	cout<<endl<<"PE 7:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[0]->showMode() ;
	pCircularPEGrid[0]->showInputVariables() ;
	pCircularPEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 8:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[1]->showMode() ;
	pCircularPEGrid[1]->showInputVariables() ;
	pCircularPEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 9:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[2]->showMode() ;
	pCircularPEGrid[2]->showInputVariables() ;
	pCircularPEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[2]->showOutputVariables();

	//Second half clock :
	pCircularPEGrid[0]->setResidentData( 0);
	pCircularPEGrid[1]->setResidentData( 0);
	pCircularPEGrid[0]->m1Input1=noElement;
	pCircularPEGrid[1]->m1Input1=noElement;

	cout<<endl<<"PE 7 (Second half clock):" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[0]->showMode() ;
	pCircularPEGrid[0]->showInputVariables() ;
	pCircularPEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 8 (Second half clock):" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[1]->showMode() ;
	pCircularPEGrid[1]->showInputVariables() ;
	pCircularPEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[1]->showOutputVariables();

    cout<<endl <<endl<<"Press the Enter key to see the next clock values...";
	cin.get();	

			break;
		
		case 11:
			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->setMode(2);
			}
			for (i=0 ; i < matrixDimension ; i++)
			{
				pCircularPEGrid[i]->setMode(1);
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->setMode(2);

			matrix[0]=noElement;
			matrix[1]=noElement;
			matrix[2]=noElement;
			unity[0]=noElement;
			unity[1]=noElement;
			unity[2]=noElement;

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->readInput();
				pCircularPEGrid[i]->readInput();
			}

			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->readInput();

			pCircularPEGrid[0]->readyForCalculation=true;
			pCircularPEGrid[1]->readyForCalculation=true;

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->doClockCalculations();	
				pCircularPEGrid[i]->doClockCalculations();	
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->doClockCalculations();	

			//Reading output and storing it in invertedMatrix :
			invertedMatrix[2][0]=pEllipticPEGrid[0][1]->outputM2output1();
	cout<<"*******************************************************************************"<<endl;
	cout<<"Clock 11:" <<endl;
	cout<<"*******************************************************************************"<<endl;
	cout<<"PE 1:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[0]->showMode() ;
	pSquarePEGrid[0]->showInputVariables() ;
	pSquarePEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 2:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][0]->showMode() ;
	pEllipticPEGrid[0][0]->showInputVariables() ;
	pEllipticPEGrid[0][0]->outputResidentData() ;
	cout<<endl <<"      ";
	pEllipticPEGrid[0][0]->showOutputVariables();

	cout<<endl<<"PE 3:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][1]->showMode() ;
	pEllipticPEGrid[0][1]->showInputVariables() ;
	pEllipticPEGrid[0][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[0][1]->showOutputVariables();

	cout<<endl<<"PE 4:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[1]->showMode() ;
	pSquarePEGrid[1]->showInputVariables() ;
	pSquarePEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 5:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[1][1]->showMode() ;
	pEllipticPEGrid[1][1]->showInputVariables() ;
	pEllipticPEGrid[1][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[1][1]->showOutputVariables();

	cout<<endl<<"PE 6:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[2]->showMode() ;
	pSquarePEGrid[2]->showInputVariables() ;
	pSquarePEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[2]->showOutputVariables();

	cout<<endl<<"PE 7:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[0]->showMode() ;
	pCircularPEGrid[0]->showInputVariables() ;
	pCircularPEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 8:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[1]->showMode() ;
	pCircularPEGrid[1]->showInputVariables() ;
	pCircularPEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 9:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[2]->showMode() ;
	pCircularPEGrid[2]->showInputVariables() ;
	pCircularPEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[2]->showOutputVariables();

    cout<<endl <<endl<<"Press the Enter key to see the next clock values...";
	cin.get();	
			break;		
		case 12:
			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->setMode(2);
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
			{
				pCircularPEGrid[i]->setMode(1);
			}

			pCircularPEGrid[2]->setMode(2);
			
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->setMode(2);

			matrix[0]=noElement;
			matrix[1]=noElement;
			matrix[2]=noElement;
			unity[0]=noElement;
			unity[1]=noElement;
			unity[2]=noElement;

			pCircularPEGrid[2]->outputUpdated=false;

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->readInput();
				pCircularPEGrid[i]->readInput();
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->readInput();

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->doClockCalculations();	
				pCircularPEGrid[i]->doClockCalculations();	
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->doClockCalculations();	

			//Reading output and storing it in invertedMatrix :
			invertedMatrix[1][0]=pEllipticPEGrid[0][0]->outputM2output1();
			invertedMatrix[2][1]=pEllipticPEGrid[0][1]->outputM2output1();
	cout<<"*******************************************************************************"<<endl;
	cout<<"Clock 12:" <<endl;
	cout<<"*******************************************************************************"<<endl;
	cout<<"PE 1:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[0]->showMode() ;
	pSquarePEGrid[0]->showInputVariables() ;
	pSquarePEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 2:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][0]->showMode() ;
	pEllipticPEGrid[0][0]->showInputVariables() ;
	pEllipticPEGrid[0][0]->outputResidentData() ;
	cout<<endl <<"      ";
	pEllipticPEGrid[0][0]->showOutputVariables();

	cout<<endl<<"PE 3:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][1]->showMode() ;
	pEllipticPEGrid[0][1]->showInputVariables() ;
	pEllipticPEGrid[0][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[0][1]->showOutputVariables();

	cout<<endl<<"PE 4:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[1]->showMode() ;
	pSquarePEGrid[1]->showInputVariables() ;
	pSquarePEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 5:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[1][1]->showMode() ;
	pEllipticPEGrid[1][1]->showInputVariables() ;
	pEllipticPEGrid[1][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[1][1]->showOutputVariables();

	cout<<endl<<"PE 6:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[2]->showMode() ;
	pSquarePEGrid[2]->showInputVariables() ;
	pSquarePEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[2]->showOutputVariables();

	cout<<endl<<"PE 7:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[0]->showMode() ;
	pCircularPEGrid[0]->showInputVariables() ;
	pCircularPEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 8:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[1]->showMode() ;
	pCircularPEGrid[1]->showInputVariables() ;
	pCircularPEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 9:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[2]->showMode() ;
	pCircularPEGrid[2]->showInputVariables() ;
	pCircularPEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[2]->showOutputVariables();

    cout<<endl <<endl<<"Press the Enter key to see the next clock values...";
	cin.get();	
			break;
		
		case 13:
			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->setMode(2);
				pCircularPEGrid[i]->setMode(2);
			}

			pCircularPEGrid[0]->setMode(1);
			pCircularPEGrid[1]->setMode(1);

			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->setMode(2);

			matrix[0]=noElement;
			matrix[1]=noElement;
			matrix[2]=noElement;
			unity[0]=noElement;
			unity[1]=noElement;
			unity[2]=noElement;

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->readInput();
				pCircularPEGrid[i]->readInput();
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->readInput();

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->doClockCalculations();	
				pCircularPEGrid[i]->doClockCalculations();	
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->doClockCalculations();	
			//Reading output and storing it in invertedMatrix :
			invertedMatrix[0][0]=pSquarePEGrid[0]->outputM2output();
			invertedMatrix[1][1]=pEllipticPEGrid[0][0]->outputM2output1();
			invertedMatrix[2][2]=pEllipticPEGrid[0][1]->outputM2output1();
	cout<<"*******************************************************************************"<<endl;
	cout<<"Clock 13:" <<endl;
	cout<<"*******************************************************************************"<<endl;
	cout<<"PE 1:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[0]->showMode() ;
	pSquarePEGrid[0]->showInputVariables() ;
	pSquarePEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 2:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][0]->showMode() ;
	pEllipticPEGrid[0][0]->showInputVariables() ;
	pEllipticPEGrid[0][0]->outputResidentData() ;
	cout<<endl <<"      ";
	pEllipticPEGrid[0][0]->showOutputVariables();

	cout<<endl<<"PE 3:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][1]->showMode() ;
	pEllipticPEGrid[0][1]->showInputVariables() ;
	pEllipticPEGrid[0][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[0][1]->showOutputVariables();

	cout<<endl<<"PE 4:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[1]->showMode() ;
	pSquarePEGrid[1]->showInputVariables() ;
	pSquarePEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 5:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[1][1]->showMode() ;
	pEllipticPEGrid[1][1]->showInputVariables() ;
	pEllipticPEGrid[1][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[1][1]->showOutputVariables();

	cout<<endl<<"PE 6:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[2]->showMode() ;
	pSquarePEGrid[2]->showInputVariables() ;
	pSquarePEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[2]->showOutputVariables();

	cout<<endl<<"PE 7:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[0]->showMode() ;
	pCircularPEGrid[0]->showInputVariables() ;
	pCircularPEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 8:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[1]->showMode() ;
	pCircularPEGrid[1]->showInputVariables() ;
	pCircularPEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 9:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[2]->showMode() ;
	pCircularPEGrid[2]->showInputVariables() ;
	pCircularPEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[2]->showOutputVariables();

    cout<<endl <<endl<<"Press the Enter key to see the next clock values...";
	cin.get();	
			break;
		case 14:
			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->setMode(2);
				pCircularPEGrid[i]->setMode(2);
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->setMode(2);

			matrix[0]=noElement;
			matrix[1]=noElement;
			matrix[2]=noElement;
			unity[0]=noElement;
			unity[1]=noElement;
			unity[2]=noElement;

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->readInput();
				pCircularPEGrid[i]->readInput();
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->readInput();

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->doClockCalculations();	
				pCircularPEGrid[i]->doClockCalculations();	
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->doClockCalculations();	

			//Reading output and storing it in invertedMatrix :
			invertedMatrix[0][1]=pSquarePEGrid[0]->outputM2output();
			invertedMatrix[1][2]=pEllipticPEGrid[0][0]->outputM2output1();
	cout<<"*******************************************************************************"<<endl;
	cout<<"Clock 14:" <<endl;
	cout<<"*******************************************************************************"<<endl;
	cout<<"PE 1:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[0]->showMode() ;
	pSquarePEGrid[0]->showInputVariables() ;
	pSquarePEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 2:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][0]->showMode() ;
	pEllipticPEGrid[0][0]->showInputVariables() ;
	pEllipticPEGrid[0][0]->outputResidentData() ;
	cout<<endl <<"      ";
	pEllipticPEGrid[0][0]->showOutputVariables();

	cout<<endl<<"PE 3:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][1]->showMode() ;
	pEllipticPEGrid[0][1]->showInputVariables() ;
	pEllipticPEGrid[0][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[0][1]->showOutputVariables();

	cout<<endl<<"PE 4:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[1]->showMode() ;
	pSquarePEGrid[1]->showInputVariables() ;
	pSquarePEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 5:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[1][1]->showMode() ;
	pEllipticPEGrid[1][1]->showInputVariables() ;
	pEllipticPEGrid[1][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[1][1]->showOutputVariables();

	cout<<endl<<"PE 6:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[2]->showMode() ;
	pSquarePEGrid[2]->showInputVariables() ;
	pSquarePEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[2]->showOutputVariables();

	cout<<endl<<"PE 7:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[0]->showMode() ;
	pCircularPEGrid[0]->showInputVariables() ;
	pCircularPEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 8:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[1]->showMode() ;
	pCircularPEGrid[1]->showInputVariables() ;
	pCircularPEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 9:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[2]->showMode() ;
	pCircularPEGrid[2]->showInputVariables() ;
	pCircularPEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[2]->showOutputVariables();

    cout<<endl <<endl<<"Press the Enter key to see the next clock values...";
	cin.get();	
			break;
		case 15:
			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->setMode(2);
				pCircularPEGrid[i]->setMode(2);
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->setMode(2);

			matrix[0]=noElement;
			matrix[1]=noElement;
			matrix[2]=noElement;
			unity[0]=noElement;
			unity[1]=noElement;
			unity[2]=noElement;

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->readInput();
				pCircularPEGrid[i]->readInput();
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->readInput();

			for (i=0 ; i < matrixDimension ; i++)
			{
				pSquarePEGrid[i]->doClockCalculations();	
				pCircularPEGrid[i]->doClockCalculations();	
			}
			for (i=0 ; i < matrixDimension-1 ; i++)
				for (j=0 ; j < matrixDimension-1 ; j++)
					if (j>=i)
						pEllipticPEGrid[i][j]->doClockCalculations();	

			//Reading output and storing it in invertedMatrix :
			invertedMatrix[0][2]=pSquarePEGrid[0]->outputM2output();
	cout<<"*******************************************************************************"<<endl;
	cout<<"Clock 15:" <<endl;
	cout<<"*******************************************************************************"<<endl;
	cout<<"PE 1:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[0]->showMode() ;
	pSquarePEGrid[0]->showInputVariables() ;
	pSquarePEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 2:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][0]->showMode() ;
	pEllipticPEGrid[0][0]->showInputVariables() ;
	pEllipticPEGrid[0][0]->outputResidentData() ;
	cout<<endl <<"      ";
	pEllipticPEGrid[0][0]->showOutputVariables();

	cout<<endl<<"PE 3:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[0][1]->showMode() ;
	pEllipticPEGrid[0][1]->showInputVariables() ;
	pEllipticPEGrid[0][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[0][1]->showOutputVariables();

	cout<<endl<<"PE 4:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[1]->showMode() ;
	pSquarePEGrid[1]->showInputVariables() ;
	pSquarePEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 5:" <<endl;
	cout<<"Mode:"<<pEllipticPEGrid[1][1]->showMode() ;
	pEllipticPEGrid[1][1]->showInputVariables() ;
	pEllipticPEGrid[1][1]->outputResidentData() ;	
	cout<<endl <<"      ";
	pEllipticPEGrid[1][1]->showOutputVariables();

	cout<<endl<<"PE 6:" <<endl;
	cout<<"Mode:"<<pSquarePEGrid[2]->showMode() ;
	pSquarePEGrid[2]->showInputVariables() ;
	pSquarePEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pSquarePEGrid[2]->showOutputVariables();

	cout<<endl<<"PE 7:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[0]->showMode() ;
	pCircularPEGrid[0]->showInputVariables() ;
	pCircularPEGrid[0]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[0]->showOutputVariables();

	cout<<endl<<"PE 8:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[1]->showMode() ;
	pCircularPEGrid[1]->showInputVariables() ;
	pCircularPEGrid[1]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[1]->showOutputVariables();

	cout<<endl<<"PE 9:" <<endl;
	cout<<"Mode:"<<pCircularPEGrid[2]->showMode() ;
	pCircularPEGrid[2]->showInputVariables() ;
	pCircularPEGrid[2]->outputResidentData() ;
	cout<<endl <<"      ";
	pCircularPEGrid[2]->showOutputVariables();
			break;
		default:{}
	}
 }

// Showing the inverted matrix:
	cout<< endl<< endl<<"This is the inverted matrix :" <<endl;
	for (i=0; i<matrixDimension; i++ )
	{
		for (j=0; j<matrixDimension; j++ )
		{
			cout.width( 15 );
			cout<< setiosflags( ios::left )<< setiosflags( ios::fixed ) \
				<<setprecision(4)<< invertedMatrix[i][j] ;
		}
		cout<< endl;
	}
 //Freeing the allocated memory in the freestore : 
	for (i=0 ; i < matrixDimension ; i++)
	{
		delete	pSquarePEGrid[i];
		delete	pCircularPEGrid[i];
	}

    for (i=0 ; i < matrixDimension-1 ; i++)
		for (j=0 ; j < matrixDimension-1 ; j++)
		{
			//Freeing the memory for upper triangle
			if (j>=i)
				delete pEllipticPEGrid[i][j];
		}
// Restarting the program:
	cout<< endl<< "Do you want to enter another matrix?(Y/N)"<< endl ;
	cin>> restart;
	if ((restart=='y') || (restart=='Y') )
		goto restartingTheProgram;

}
